@@ -12,6 +12,7 @@ import websockets as ws
 from .client import AsyncClient
 from .exceptions import BinanceWebsocketUnableToConnect
 from .enums import FuturesType
+from .enums import ContractType
 from .threaded_stream import ThreadedApiManager
 
 KEEPALIVE_TIMEOUT = 5 * 60  # 5 minutes
@@ -24,6 +25,14 @@ class WSListenerState(Enum):
     EXITING = 'Exiting'
 
 
+class BinanceSocketType(str, Enum):
+    SPOT = 'Spot'
+    USD_M_FUTURES = 'USD_M_Futures'
+    COIN_M_FUTURES = 'Coin_M_Futures'
+    OPTIONS = 'Vanilla_Options'
+    ACCOUNT = 'Account'
+
+
 class ReconnectingWebsocket:
 
     MAX_RECONNECTS = 5
@@ -67,7 +76,7 @@ class ReconnectingWebsocket:
         await self._before_connect()
         assert self._path
         ws_url = self._url + self._prefix + self._path
-        self._conn = ws.connect(ws_url)
+        self._conn = ws.connect(ws_url, close_timeout=0.001)
         try:
             self.ws = await self._conn.__aenter__()
         except:  # noqa
@@ -76,7 +85,7 @@ class ReconnectingWebsocket:
         self.ws_state = WSListenerState.STREAMING
         self._reconnects = 0
         await self._after_connect()
-        self._loop.call_soon(asyncio.create_task, self._read_loop())
+        self._loop.call_soon_threadsafe(asyncio.create_task, self._read_loop())
 
     async def _before_connect(self):
         pass
@@ -119,14 +128,14 @@ class ReconnectingWebsocket:
             try:
                 res = await asyncio.wait_for(self.ws.recv(), timeout=self.TIMEOUT)
             except asyncio.TimeoutError:
-                logging.debug(f"no message in {self.TIMEOUT} seconds")
+                self._log.debug(f"no message in {self.TIMEOUT} seconds")
             except asyncio.CancelledError as e:
-                logging.debug(f"cancelled error {e}")
+                self._log.debug(f"cancelled error {e}")
                 break
             except asyncio.IncompleteReadError as e:
-                logging.debug(f"incomplete read error {e}")
+                self._log.debug(f"incomplete read error {e}")
             except Exception as e:
-                logging.debug(f"exception {e}")
+                self._log.debug(f"exception {e}")
                 break
             else:
                 if self.ws_state in (WSListenerState.EXITING, WSListenerState.RECONNECTING):
@@ -144,16 +153,16 @@ class ReconnectingWebsocket:
             try:
                 res = await asyncio.wait_for(self._queue.get(), timeout=self.TIMEOUT)
             except asyncio.TimeoutError:
-                logging.debug(f"no message in {self.TIMEOUT} seconds")
+                self._log.debug(f"no message in {self.TIMEOUT} seconds")
         return res
 
     async def _wait_for_reconnect(self):
         while self.ws_state == WSListenerState.RECONNECTING:
-            logging.debug("reconnecting waiting for connect")
+            self._log.debug("reconnecting waiting for connect")
         if not self.ws:
-            logging.debug("ignore message no ws")
+            self._log.debug("ignore message no ws")
         else:
-            logging.debug(f"ignore message {self.ws_state}")
+            self._log.debug(f"ignore message {self.ws_state}")
 
     def _get_reconnect_wait(self, attempts: int) -> int:
         expo = 2 ** attempts
@@ -166,7 +175,7 @@ class ReconnectingWebsocket:
         self._reconnects += 1
 
     def _no_message_received_reconnect(self):
-        logging.debug('No message received, reconnecting')
+        self._log.debug('No message received, reconnecting')
         asyncio.create_task(self._reconnect())
 
     async def _reconnect(self):
@@ -176,20 +185,23 @@ class ReconnectingWebsocket:
         await self.before_reconnect()
         if self._reconnects < self.MAX_RECONNECTS:
             reconnect_wait = self._get_reconnect_wait(self._reconnects)
-            logging.debug(
+            self._log.debug(
                 f"websocket reconnecting {self.MAX_RECONNECTS - self._reconnects} reconnects left - "
                 f"waiting {reconnect_wait}"
             )
             await asyncio.sleep(reconnect_wait)
             await self.connect()
         else:
-            logging.error(f'Max reconnections {self.MAX_RECONNECTS} reached:')
+            self._log.error(f'Max reconnections {self.MAX_RECONNECTS} reached:')
             raise BinanceWebsocketUnableToConnect
 
 
 class KeepAliveWebsocket(ReconnectingWebsocket):
 
-    def __init__(self, client: AsyncClient, loop, url, keepalive_type, prefix='ws/', is_binary=False, exit_coro=None, user_timeout=None):
+    def __init__(
+        self, client: AsyncClient, loop, url, keepalive_type, prefix='ws/', is_binary=False, exit_coro=None,
+        user_timeout=None
+    ):
         super().__init__(loop=loop, path=None, url=url, prefix=prefix, is_binary=is_binary, exit_coro=exit_coro)
         self._keepalive_type = keepalive_type
         self._client = client
@@ -236,11 +248,11 @@ class KeepAliveWebsocket(ReconnectingWebsocket):
         listen_key = await self._get_listen_key()
 
         if listen_key != self._path:
-            logging.debug("listen key changed: reconnect")
+            self._log.debug("listen key changed: reconnect")
             self._path = listen_key
             await self._reconnect()
         else:
-            logging.debug("listen key same: keepalive")
+            self._log.debug("listen key same: keepalive")
             if self._keepalive_type == 'user':
                 await self._client.stream_keepalive(self._path)
             elif self._keepalive_type == 'margin':  # cross-margin
@@ -298,9 +310,13 @@ class BinanceSocketManager:
             stream_url = self.STREAM_TESTNET_URL
         return stream_url
 
-    def _get_socket(self, path: str, stream_url: Optional[str] = None, prefix: str = 'ws/', is_binary: bool = False):
-        if path not in self._conns:
-            self._conns[path] = ReconnectingWebsocket(
+    def _get_socket(
+        self, path: str, stream_url: Optional[str] = None, prefix: str = 'ws/', is_binary: bool = False,
+        socket_type: BinanceSocketType = BinanceSocketType.SPOT
+    ) -> str:
+        conn_id = f'{socket_type}_{path}'
+        if conn_id not in self._conns:
+            self._conns[conn_id] = ReconnectingWebsocket(
                 loop=self._loop,
                 path=path,
                 url=self._get_stream_url(stream_url),
@@ -309,13 +325,14 @@ class BinanceSocketManager:
                 is_binary=is_binary
             )
 
-        return self._conns[path]
+        return self._conns[conn_id]
 
     def _get_account_socket(
         self, path: str, stream_url: Optional[str] = None, prefix: str = 'ws/', is_binary: bool = False
     ):
-        if path not in self._conns:
-            self._conns[path] = KeepAliveWebsocket(
+        conn_id = f'{BinanceSocketType.ACCOUNT}_{path}'
+        if conn_id not in self._conns:
+            self._conns[conn_id] = KeepAliveWebsocket(
                 client=self._client,
                 loop=self._loop,
                 url=self._get_stream_url(stream_url),
@@ -326,9 +343,10 @@ class BinanceSocketManager:
                 user_timeout=self._user_timeout
             )
 
-        return self._conns[path]
+        return self._conns[conn_id]
 
     def _get_futures_socket(self, path: str, futures_type: FuturesType, prefix: str = 'stream?streams='):
+        socket_type: BinanceSocketType = BinanceSocketType.USD_M_FUTURES
         if futures_type == FuturesType.USD_M:
             stream_url = self.FSTREAM_URL
             if self.testnet:
@@ -337,13 +355,13 @@ class BinanceSocketManager:
             stream_url = self.DSTREAM_URL
             if self.testnet:
                 stream_url = self.DSTREAM_TESTNET_URL
-        return self._get_socket(path, stream_url, prefix)
+        return self._get_socket(path, stream_url, prefix, socket_type=socket_type)
 
     def _get_options_socket(self, path: str, prefix: str = 'ws/'):
         stream_url = self.VSTREAM_URL
         if self.testnet:
             stream_url = self.VSTREAM_TESTNET_URL
-        return self._get_socket(path, stream_url, prefix, is_binary=True)
+        return self._get_socket(path, stream_url, prefix, is_binary=True, socket_type=BinanceSocketType.OPTIONS)
 
     async def _exit_socket(self, path: str):
         await self._stop_socket(path)
@@ -466,6 +484,56 @@ class BinanceSocketManager:
         path = f'{symbol.lower()}@kline_{interval}'
         return self._get_socket(path)
 
+    def kline_futures_socket(self, symbol: str, interval=AsyncClient.KLINE_INTERVAL_1MINUTE,
+                             futures_type: FuturesType = FuturesType.USD_M,
+                             contract_type: ContractType = ContractType.PERPETUAL):
+        """Start a websocket for symbol kline data for the perpeual futures stream
+
+        https://binance-docs.github.io/apidocs/futures/en/#continuous-contract-kline-candlestick-streams
+
+        :param symbol: required
+        :type symbol: str
+        :param interval: Kline interval, default KLINE_INTERVAL_1MINUTE
+        :type interval: str
+        :param futures_type: use USD-M or COIN-M futures default USD-M
+        :param contract_type: use PERPETUAL or CURRENT_QUARTER or NEXT_QUARTER default PERPETUAL
+
+        :returns: connection key string if successful, False otherwise
+
+        Message Format
+
+        .. code-block:: python
+
+                {
+                "e":"continuous_kline",   // Event type
+                "E":1607443058651,        // Event time
+                "ps":"BTCUSDT",           // Pair
+                "ct":"PERPETUAL"          // Contract type
+                "k":{
+                    "t":1607443020000,      // Kline start time
+                    "T":1607443079999,      // Kline close time
+                    "i":"1m",               // Interval
+                    "f":116467658886,       // First trade ID
+                    "L":116468012423,       // Last trade ID
+                    "o":"18787.00",         // Open price
+                    "c":"18804.04",         // Close price
+                    "h":"18804.04",         // High price
+                    "l":"18786.54",         // Low price
+                    "v":"197.664",          // volume
+                    "n": 543,               // Number of trades
+                    "x":false,              // Is this kline closed?
+                    "q":"3715253.19494",    // Quote asset volume
+                    "V":"184.769",          // Taker buy volume
+                    "Q":"3472925.84746",    //Taker buy quote asset volume
+                    "B":"0"                 // Ignore
+                }
+            }
+            <pair>_<contractType>@continuousKline_<interval>
+        """
+
+        path = f'{symbol.lower()}_{contract_type}@continuousKline_{interval}'
+        return self._get_futures_socket(path, futures_type=futures_type)
+
     def miniticker_socket(self, update_time: int = 1000):
         """Start a miniticker websocket for all trades
 
@@ -592,7 +660,7 @@ class BinanceSocketManager:
     def symbol_miniticker_socket(self, symbol: str):
         """Start a websocket for a symbol's miniTicker data
 
-                https://github.com/binance/binance-spot-api-docs/blob/master/web-socket-streams.md#individual-symbol-mini-ticker-stream
+                https://binance-docs.github.io/apidocs/spot/en/#individual-symbol-mini-ticker-stream
 
                 :param symbol: required
                 :type symbol: str
@@ -937,11 +1005,23 @@ class BinanceSocketManager:
         path = f'streams={"/".join(streams)}'
         return self._get_futures_socket(path, prefix='stream?', futures_type=futures_type)
 
+    def futures_depth_socket(self, symbol: str, depth: str = '10', futures_type: FuturesType = FuturesType.USD_M):
+        """Subscribe to a depth data stream
+
+        https://binance-docs.github.io/apidocs/futures/en/#partial-book-depth-streams
+
+        :param symbol: required
+        :type symbol: str
+        :param depth: optional Number of depth entries to return, default 10.
+        :type depth: str
+        """
+        return self._get_futures_socket(symbol.lower() + '@depth' + str(depth), futures_type=futures_type)
+
     def user_socket(self):
         """Start a websocket for user data
 
-            https://github.com/binance-exchange/binance-official-api-docs/blob/master/user-data-stream.md
-            https://binance-docs.github.io/apidocs/spot/en/#listen-key-spot
+        https://github.com/binance-exchange/binance-official-api-docs/blob/master/user-data-stream.md
+        https://binance-docs.github.io/apidocs/spot/en/#listen-key-spot
 
         :returns: connection key string if successful, False otherwise
 
@@ -949,6 +1029,18 @@ class BinanceSocketManager:
         """
         return self._get_account_socket('user')
 
+    def futures_user_socket(self):
+        """Start a websocket for coin futures user data
+
+        https://binance-docs.github.io/apidocs/futures/en/#user-data-streams
+
+        :returns: connection key string if successful, False otherwise
+
+        Message Format - see Binanace API docs for all types
+        """
+
+        return self._get_account_socket('futures', stream_url=self.FSTREAM_URL)
+
     def margin_socket(self):
         """Start a websocket for cross-margin data
 
@@ -1066,7 +1158,7 @@ class ThreadedWebsocketManager(ThreadedApiManager):
 
     async def _before_socket_listener_start(self):
         assert self._client
-        self._bsm = BinanceSocketManager(self._client, self._loop)
+        self._bsm = BinanceSocketManager(client=self._client, loop=self._loop)
 
     def _start_async_socket(
         self, callback: Callable, socket_name: str, params: Dict[str, Any], path: Optional[str] = None
@@ -1076,7 +1168,7 @@ class ThreadedWebsocketManager(ThreadedApiManager):
         socket = getattr(self._bsm, socket_name)(**params)
         path = path or socket._path  # noqa
         self._socket_running[path] = True
-        self._loop.call_soon(asyncio.create_task, self.start_listener(socket, socket._path, callback))
+        self._loop.call_soon_threadsafe(asyncio.create_task, self.start_listener(socket, socket._path, callback))
         return path
 
     def start_depth_socket(
@@ -1102,6 +1194,21 @@ class ThreadedWebsocketManager(ThreadedApiManager):
             }
         )
 
+    def start_kline_futures_socket(self, callback: Callable, symbol: str,
+                                   interval=AsyncClient.KLINE_INTERVAL_1MINUTE,
+                                   futures_type: FuturesType = FuturesType.USD_M,
+                                   contract_type: ContractType = ContractType.PERPETUAL) -> str:
+        return self._start_async_socket(
+            callback=callback,
+            socket_name='kline_futures_socket',
+            params={
+                'symbol': symbol,
+                'interval': interval,
+                'futures_type': futures_type.value,
+                'contract_type': contract_type.value
+            }
+        )
+
     def start_miniticker_socket(self, callback: Callable, update_time: int = 1000) -> str:
         return self._start_async_socket(
             callback=callback,
